<?php

class Core {

	############################################
	# DATABASE CONNECTION
	############################################	

	function dblConnect () {		
		try {   
			$this->dbl = new PDO('mysql:host='.DBL_HOST.';dbname='.DBL_NAME, DBL_USER, DBL_PASS);
			$this->dbl->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$this->dbl->setAttribute(PDO::ATTR_EMULATE_PREPARES, true);
			$this->dbl->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true); 	
			$this->dbl->query("SET NAMES utf8");		
		} catch(PDOException $e){
			DEBUG ? die('DATABASE CONNECTION ERROR: ' . $e->getMessage()) : die();
		}
	}

	function dblDisconnect () {	
		unset($this->dbl);
	}

	
	############################################
	# CORE FUNCTIONS
	############################################
	
	function dehashIP($ipnum) {
		return long2ip($ipnum);
	}
	
	############################################
	# GENERATOR FUNCTIONS
	############################################
	
	function getMacForNode($nodeid) 
	{
		try{
			$sql = "
				SELECT
					`mac`
				FROM
					`macs`
				WHERE
					`nodeid` = :nodeid
				";
				
			$stmt = $this->dbl->prepare($sql);
			$stmt->bindValue(':nodeid', $nodeid, PDO::PARAM_INT);	
   			$stmt->execute();
			while($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$retval[] = $result['mac'];
			}
			$stmt->closeCursor();
			return implode(",", $retval);
		} catch(PDOException $e){
			DEBUG ? die('SQL Error: '.$e->getMessage()) : die();
		}
	}
	
	function getMacForApc($netdev) 
	{
		try{
			$sql = "
				SELECT
					`macs`.`mac`
				FROM
					`nodes`
				LEFT JOIN
					`macs`
				ON
					`macs`.`nodeid` = `nodes`.`id`
				WHERE
					`nodes`.`netdev` = :netdev
				AND
					`nodes`.`ownerid` = 0
				";
				
			$stmt = $this->dbl->prepare($sql);
			$stmt->bindValue(':netdev', $netdev, PDO::PARAM_INT);	
   			$stmt->execute();
			while($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$retval[] = $result['mac'];
			}
			$stmt->closeCursor();
			return implode(",", $retval);
		} catch(PDOException $e){
			DEBUG ? die('SQL Error: '.$e->getMessage()) : die();
		}
	}	
	
	function getNodes() 
	{
		try{
			$sql = "
				SELECT
					`nodes`.*,
					`netdevices`.`nastype`,
					`nodegroups`.`description` AS majorband,
					`customers`.`pin`,
					CONCAT(`tariffs`.`downceil`,'/',`tariffs`.`upceil`) AS `minorband`,
					CONCAT(`tariffs`.`downceil_n`,'/',`tariffs`.`upceil_n`) AS `minorband_n`
				FROM
					`nodes`
				LEFT JOIN
					`customers`
				ON
					`customers`.`id` = `nodes`.`ownerid`
				LEFT JOIN
					`netdevices`
				ON
					`netdevices`.`id` = `nodes`.`netdev`
				LEFT JOIN
					`nodegroupassignments`
				ON
					`nodegroupassignments`.`nodeid` = `nodes`.`id`
				LEFT JOIN
					`nodegroups`
				ON
					`nodegroups`.`id` = `nodegroupassignments`.`nodegroupid`
				LEFT JOIN
					`nodeassignments`
				ON
					`nodeassignments`.`nodeid` = `nodes`.`id`
				LEFT JOIN
					`assignments`
				ON
					`assignments`.`id` = `nodeassignments`.`assignmentid`
				LEFT JOIN
					`tariffs`
				ON
					`tariffs`.`id` = `assignments`.`tariffid`							
				WHERE
					`nodes`.`ownerid` > 0
				ORDER BY
					`nodes`.`ipaddr` ASC
				";
				
			$stmt = $this->dbl->prepare($sql);	
   			$stmt->execute();
			while($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
				foreach($result as $k => $v) {
					$retval[$result['id']][$k] = trim(iconv('ISO-8859-2', 'UTF-8', $v));
				}
				$retval[$result['id']]['ipaddr'] = $this->dehashIP($retval[$result['id']]['ipaddr']);
				$retval[$result['id']]['ipaddr_pub'] = $this->dehashIP($retval[$result['id']]['ipaddr_pub']);
				$retval[$result['id']]['dhcpmac'] = $this->getMacForNode($result['id']);
				if($retval[$result['id']]['nastype'] == 1000000 && $retval[$result['id']]['netdev']) {
					$retval[$result['id']]['authmac'] = $this->getMacForApc($retval[$result['id']]['netdev']);
				} else {
					$retval[$result['id']]['authmac'] = $retval[$result['id']]['dhcpmac'];
				}
			}
			$stmt->closeCursor();
			return $retval;	
		} catch(PDOException $e){
			DEBUG ? die('SQL Error: '.$e->getMessage()) : die();
		}
	}
	
	function generateConfig($nodes) 
	{
		global $cf;
		$config = "# IP\t\tDOWN\tUP\tNDOWN\tNUP\tBLOK\tAUTHMAC\n\n";
		if ($nodes) foreach($nodes as $k => $v) {
			if($v['majorband']) {
				$band = explode("/", $v['majorband']);
				$band_n = explode("/", $v['majorband']);
			} elseif($v['minorband']) {
				$band = explode("/", $v['minorband']);
				$band_n = explode("/", $v['minorband_n']);
			} else {
				$band[0] = 512; $band[1] = 128;
				$band_n[0] = 512; $band_n[1] = 128;
			}
			if($v['access'] == 0) {
				$block = 2;
			} elseif($v['warning']) {
				$block = 1;
			} else {
				$block = 0;
			}
			$band_n[0] ? null : $band_n[0] = $band[0];
			$band_n[1] ? null : $band_n[1] = $band[1];
			$config .= 
				$v['ipaddr']."\t".
				$band[0]."\t".
				$band[1]."\t".
				$band_n[0]."\t".
				$band_n[1]."\t".
				$block."\t".
				$v['authmac']."\n";
		}
		if($config) {
			if(file_exists($cf)) {
			    $fp = fopen($cf, "r"); $config_old = fread($fp, filesize($cf)); fclose($fp);
			}
			if(trim($config_old) != trim($config)) {
				$fp = fopen($cf, 'w'); fputs($fp, $config); fclose($fp);
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	function generatePubConfig($nodes) 
	{
		global $pub;
		$config = "#PUBIP\tIP\n\n";
		if ($nodes) foreach($nodes as $k => $v) if($v['ipaddr_pub'] != "0.0.0.0") $config .= $v['ipaddr_pub']."\t".$v['ipaddr']."\n";
		if($config) {
			if(file_exists($pub)) {
			    $fp = fopen($pub, "r"); $config_old = fread($fp, filesize($pub)); fclose($fp);
			}
			if(trim($config_old) != trim($config)) {
				$fp = fopen($pub, 'w'); fputs($fp, $config); fclose($fp);
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	function generatePppoeConfig($nodes) 
	{
		global $pppoe;
		$config = '';
		if ($nodes) foreach($nodes as $k => $v) {
			$v['passwd'] ? $pass = $v['passwd'] : $pass = $v['pin'];
			$config .= "\"".$v['name']."\"\t\t*\t\t\"".$pass."\"\t\t".$v['ipaddr']."\n";
		}
		
		if($config) {
			$fp = fopen($pppoe, "r"); $config_old = fread($fp, filesize($pppoe)); fclose($fp);
			if(trim($config_old) != trim($config)) {
				$fp = fopen($pppoe, 'w'); fputs($fp, $config); fclose($fp);
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}	

	function whatToDo() 
	{
		try{
			$sql = "
				SELECT
					*
				FROM
					`vsmart`
				";
				
			$stmt = $this->dbl->prepare($sql);	
   			$stmt->execute();
			while($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$retval[$result['command']] = $result['value'];
			}
			$stmt->closeCursor();
			return $retval;
		} catch(PDOException $e){
			DEBUG ? die('SQL Error: '.$e->getMessage()) : die();
		}
	}
	
	function updateApi($command, $value) 
	{
		try{
			$sql = "
				UPDATE
					`vsmart`
				SET
					`value` = :value
				WHERE
					`command` = :command
				LIMIT 1;
				";
				
			$stmt = $this->dbl->prepare($sql);
			$stmt->bindValue(':command', $command, PDO::PARAM_STR);
			$stmt->bindValue(':value', $value, PDO::PARAM_STR);		
   			$stmt->execute();
			$stmt->closeCursor();
			return true;
		} catch(PDOException $e){
			DEBUG ? die('SQL Error: '.$e->getMessage()) : die();
		}
	}	
		
}
 
?>